@@ -1018,6 +1018,14 @@ static int get_search_criterion(struct protstream *pin,
     case '0': case '1': case '2': case '3': case '4':
     case '5': case '6': case '7': case '8': case '9':
     case '*':                                           /* RFC 3501 */
+        if (client_capa & CAPA_UIDONLY) {
+            prot_printf(pout,
+                        "%s BAD [UIDREQUIRED] Message numbers are not allowed in"
+                        " Search after UIDONLY is enabled\r\n", base->tag);
+            if (c != EOF) prot_ungetc(c, pin);
+            return EOF;
+        }
+
         if (imparse_issequence(criteria.s)) {
             seqset_t *seq;
             e = search_expr_new(parent, SEOP_MATCH);
@@ -1239,7 +1247,7 @@ static int get_search_criterion(struct protstream *pin,
         break;
 
     case 'm':
-        if (!strcmp(criteria.s, "modseq")) {            /* RFC 4551 */
+        if (!strcmp(criteria.s, "modseq")) {            /* RFC 7162 */
             modseq_t modseq;
             if (c != ' ') goto missingarg;
             /* Check for optional search-modseq-ext */
@@ -1254,6 +1262,8 @@ static int get_search_criterion(struct protstream *pin,
             e = search_expr_new(parent, SEOP_GE);
             e->attr = search_attr_find("modseq");
             e->value.u = modseq;
+
+            base->state |= GETSEARCH_MODSEQ;  // CONDSTORE enabling command
         }
         else goto badcri;
         break;
