@@ -142,8 +142,8 @@ static struct msg {
     uint32_t uid;
     uint32_t recno;
     uint32_t size;
-    int deleted:1;
-    int seen:1;
+    unsigned int deleted:1;
+    unsigned int seen:1;
 } *popd_map = NULL;
 
 static struct io_count *io_count_start;
@@ -156,6 +156,8 @@ static int popd_tls_required = 0;
 
 static int popd_myrights;
 
+static struct proc_handle *proc_handle = NULL;
+
 /* the sasl proxy policy context */
 static struct proxy_context popd_proxyctx = {
     0, 1, &popd_authstate, NULL, NULL
@@ -328,7 +330,7 @@ static void popd_reset(void)
     uint64_t bytes_in = 0;
     uint64_t bytes_out = 0;
 
-    proc_cleanup();
+    proc_cleanup(&proc_handle);
 
     syslog(LOG_NOTICE, "counts: retr=<%d> top=<%d> dele=<%d>",
                        count_retr, count_top, count_dele);
@@ -416,7 +418,7 @@ int service_init(int argc __attribute__((unused)),
     int opt;
 
     if (geteuid() == 0) fatal("must run as the Cyrus user", EX_USAGE);
-    setproctitle_init(argc, argv, envp);
+    proc_settitle_init(argc, argv, envp);
 
     /* set signal handlers */
     signals_set_shutdown(&shut_down);
@@ -429,7 +431,7 @@ int service_init(int argc __attribute__((unused)),
     idle_init();
 
     /* Set namespace */
-    if ((r = mboxname_init_namespace(&popd_namespace, 1)) != 0) {
+    if ((r = mboxname_init_namespace(&popd_namespace, NAMESPACE_OPTION_ADMIN))) {
         syslog(LOG_ERR, "%s", error_message(r));
         fatal(error_message(r), EX_CONFIG);
     }
@@ -606,7 +608,7 @@ void shut_down(int code)
 
     libcyrus_run_delayed();
 
-    proc_cleanup();
+    proc_cleanup(&proc_handle);
 
     /* close local mailbox */
     if (popd_mailbox)
@@ -678,7 +680,7 @@ EXPORTED void fatal(const char* s, int code)
 
     if (recurse_code) {
         /* We were called recursively. Just give up */
-        proc_cleanup();
+        proc_cleanup(&proc_handle);
         exit(recurse_code);
     }
     recurse_code = code;
@@ -857,12 +859,20 @@ static void cmdloop(void)
     char *p;
     char *arg;
     uint32_t msgno = 0;
+    int r;
 
     for (;;) {
         signals_poll();
 
         /* register process */
-        proc_register(config_ident, popd_clienthost, popd_userid, popd_mailbox ? mailbox_name(popd_mailbox) : NULL, NULL);
+        r = proc_register(&proc_handle, 0,
+                          config_ident, popd_clienthost, popd_userid,
+                          popd_mailbox ? mailbox_name(popd_mailbox) : NULL,
+                          NULL);
+        if (r) fatal("unable to register process", EX_IOERR);
+        proc_settitle(config_ident, popd_clienthost, popd_userid,
+                      popd_mailbox ? mailbox_name(popd_mailbox) : NULL,
+                      NULL);
 
         libcyrus_run_delayed();
 
@@ -941,7 +951,14 @@ static void cmdloop(void)
             syslog(LOG_NOTICE, "command: %s", inputbuf);
 
         /* register process */
-        proc_register(config_ident, popd_clienthost, popd_userid, popd_mailbox ? mailbox_name(popd_mailbox) : NULL, inputbuf);
+        r = proc_register(&proc_handle, 0,
+                          config_ident, popd_clienthost, popd_userid,
+                          popd_mailbox ? mailbox_name(popd_mailbox) : NULL,
+                          inputbuf);
+        if (r) fatal("unable to register process", EX_IOERR);
+        proc_settitle(config_ident, popd_clienthost, popd_userid,
+                      popd_mailbox ? mailbox_name(popd_mailbox) : NULL,
+                      inputbuf);
 
         if (!strcmp(inputbuf, "quit")) {
             if (!arg) {
@@ -2125,7 +2142,7 @@ static void bitpipe(void)
             goto done;
         }
     } while (!proxy_check_input(protin, popd_in, popd_out,
-                                backend->in, backend->out, 0));
+                                backend->in, backend->out, PROT_NO_FD, NULL, 0));
 
  done:
     /* ok, we're done. */
